@@ -85,21 +85,25 @@ SIEGE_MACHINE_ORDER = [
     "Battle Blimp",
     "Stone Slammer",
     "Siege Barracks",
+    "Log Launcher",
 ]
 
 SUPER_TROOP_ORDER = [
     "Super Barbarian",
+    "Super Archer",
     "Super Giant",
     "Sneaky Goblin",
     "Super Wall Breaker",
+    "Super Wizard",
     "Inferno Dragon",
+    "Super Minion",
+    "Super Valkyrie",
     "Super Witch",
-    "Super Archer",  # not added yet but they're in the API payload for some reason
-    "Super Valkyrie",  # not added yet but they're in the API payload for some reason
+    "Ice Hound",
 ]
 
-# TODO: when SC fixes Super Troops in API add them here
-HOME_TROOP_ORDER = ELIXIR_TROOP_ORDER + DARK_ELIXIR_TROOP_ORDER + SIEGE_MACHINE_ORDER
+# TODO: When v2 comes around, remove siege machines from here
+HOME_TROOP_ORDER = ELIXIR_TROOP_ORDER + SUPER_TROOP_ORDER + DARK_ELIXIR_TROOP_ORDER + SIEGE_MACHINE_ORDER
 
 BUILDER_TROOPS_ORDER = [
     "Raged Barbarian",
@@ -122,6 +126,7 @@ ELIXIR_SPELL_ORDER = [
     "Jump Spell",
     "Freeze Spell",
     "Clone Spell",
+    "Invisibility Spell",
 ]
 
 DARK_ELIXIR_SPELL_ORDER = [
@@ -136,44 +141,53 @@ SPELL_ORDER = ELIXIR_SPELL_ORDER + DARK_ELIXIR_SPELL_ORDER
 
 HERO_ORDER = ["Barbarian King", "Archer Queen", "Grand Warden", "Royal Champion", "Battle Machine"]
 
+HERO_PETS_ORDER = ["L.A.S.S.I", "Electro Owl", "Mighty Yak", "Unicorn"]
+
 ACHIEVEMENT_ORDER = [
-    "Bigger Coffers",
-    "Get those Goblins!",
+    # Home Base
+    "Keep Your Account Safe!",
     "Bigger & Better",
-    "Nice and Tidy",
-    "Release the Beasts",
+    "Discover New Troops",
+    "Bigger Coffers",
     "Gold Grab",
     "Elixir Escapade",
-    "Sweet Victory!",
-    "Empire Builder",
-    "Wall Buster",
-    "Humiliator",
-    "Union Buster",
-    "Conqueror",
-    "Unbreakable",
-    "Friend in Need",
-    "Mortar Mauler",
     "Heroic Heist",
+    "Well Seasoned",
+    "Nice and Tidy",
+    "Empire Builder",
+    "Clan War Wealth",
+    "Friend in Need",
+    "Sharing is caring",
+    "Siege Sharer",
+    "War Hero",
+    "War League Legend",
+    "Games Champion",
+    "Unbreakable",
+    "Sweet Victory!",
+    "Conqueror",
     "League All-Star",
+    "Humiliator",
+    "Not So Easy This Time",
+    "Union Buster",
+    "Bust This!",
+    "Wall Buster",
+    "Mortar Mauler",
     "X-Bow Exterminator",
     "Firefighter",
-    "War Hero",
-    "Treasurer",
     "Anti-Artillery",
-    "Sharing is caring",
-    "Games Champion",
+    "Shattered and Scattered",
+    "Get those Goblins!",
     "Get those other Goblins!",
     "Dragon Slayer",
-    "War League Legend",
-    "Keep your village safe",
-    "Well Seasoned",
-    "Shattered and Scattered",
+    "Superb Work",
+
+    # Builder Base
     "Master Engineering",
+    "Hidden Treasures",
+    "High Gear",
     "Next Generation Model",
     "Un-Build It",
     "Champion Builder",
-    "High Gear",
-    "Hidden Treasures",
 ]
 
 UNRANKED_LEAGUE_DATA = {
