@@ -202,9 +202,7 @@ class HTTPClient:
         else:
             raise TypeError("throttler must be either BasicThrottler or BatchThrottler.")
 
-        self.__session = aiohttp.ClientSession(
-            loop=self.loop, connector=connector, timeout=aiohttp.ClientTimeout(total=timeout)
-        )
+        self.__session = aiohttp.ClientSession(connector=connector, timeout=aiohttp.ClientTimeout(total=timeout))
 
         self._keys = None  # defined in get_keys()
         self.keys = None  # defined in get_keys()
@@ -215,45 +213,6 @@ class HTTPClient:
         except KeyError:
             pass
 
-    async def get_keys(self):
-        self.client._ready.clear()
-
-        key_count = self.key_count
-        response_dict, session = await self.login_to_site(self.email, self.password)
-        cookies = self.create_cookies(response_dict, session)
-        current_keys = (await self.find_site_keys(cookies))["keys"]
-        self._keys = [key["key"] for key in current_keys if key["name"] == self.key_names]
-        available_keys = KEY_MAXIMUM - len(current_keys)
-
-        if len(self._keys) <= key_count:
-            keys_needed = key_count - len(self._keys)
-            if available_keys >= keys_needed:
-                ip_ = await self.get_ip()
-                for _ in range(keys_needed):
-                    key_description = "Created on {}".format(datetime.now().strftime("%c"))
-                    self._keys.append(
-                        await self.create_key(cookies, self.key_names, key_description, [ip_], [self.key_scopes])
-                    )
-            else:
-                await self.close()
-                raise RuntimeError(
-                    (
-                        "There are {} API keys already created and "
-                        '{} match "{}" out of a max of {} please goto {} '
-                        'and delete unused keys or rename to "{}".'
-                    ).format(
-                        len(current_keys),
-                        len(self._keys),
-                        self.key_names,
-                        KEY_MAXIMUM,
-                        "https://developer.clashofclans.com",
-                        self.key_names,
-                    )
-                )
-
-        self.keys = cycle(self._keys)
-        self.client._ready.set()
-
     async def close(self):
         if self.__session:
             await self.__session.close()
@@ -283,137 +242,95 @@ class HTTPClient:
         cache_control_key = route.cache_control_key
         cache = self.cache
         # the cache will be cleaned once it becomes stale / a new object is available from the api.
-        if cache:
+        if cache is not None:
             try:
                 return cache[cache_control_key]
             except KeyError:
                 pass
 
-        async with self.__lock, self.__throttle:
-            start = perf_counter()
-            async with self.__session.request(method, url, **kwargs) as response:
-                perfcounter = (perf_counter() - start) * 1000
-                log_info = {"method": method, "url": url, "perf_counter": perfcounter, "status": response.status}
-                self.stats[url] = perfcounter
-                LOG.debug("API HTTP Request: %s", str(log_info))
-                data = await json_or_text(response)
+        for tries in range(5):
+            try:
+                async with self.__lock, self.__throttle:
+                    start = perf_counter()
+                    async with self.__session.request(method, url, **kwargs) as response:
+                        perfcounter = (perf_counter() - start) * 1000
+                        log_info = {"method": method, "url": url, "perf_counter": perfcounter, "status": response.status}
+                        self.stats[url] = perfcounter
+                        LOG.debug("API HTTP Request: %s", str(log_info))
+                        data = await json_or_text(response)
 
-                try:
-                    # set a callback to remove the item from cache once it's stale.
-                    delta = int(response.headers["Cache-Control"].strip("max-age="))
-                    data["_response_retry"] = delta
-                    if cache:
-                        self.cache[cache_control_key] = data
-                        LOG.debug("Cache-Control max age: %s seconds, key: %s", delta, cache_control_key)
-                        self.loop.call_later(delta, self._cache_remove, cache_control_key)
+                        try:
+                            # set a callback to remove the item from cache once it's stale.
+                            delta = int(response.headers["Cache-Control"].strip("max-age="))
+                            data["_response_retry"] = delta
+                            if cache is not None:
+                                self.cache[cache_control_key] = data
+                                LOG.debug("Cache-Control max age: %s seconds, key: %s", delta, cache_control_key)
+                                self.loop.call_later(delta, self._cache_remove, cache_control_key)
 
-                except (KeyError, AttributeError, ValueError):
-                    # the request didn't contain cache control headers so skip any cache handling.
-                    # if the API returns a timeout error (504) it will return a string of HTML.
-                    if isinstance(data, dict):
-                        data["_response_retry"] = 0
+                        except (KeyError, AttributeError, ValueError):
+                            # the request didn't contain cache control headers so skip any cache handling.
+                            # if the API returns a timeout error (504) it will return a string of HTML.
+                            if isinstance(data, dict):
+                                data["_response_retry"] = 0
 
-                if 200 <= response.status < 300:
-                    LOG.debug("%s has received %s", url, data)
-                    return data
+                        if 200 <= response.status < 300:
+                            LOG.debug("%s has received %s", url, data)
+                            return data
 
-                if response.status == 400:
-                    raise InvalidArgument(response, data)
+                        if response.status == 400:
+                            raise InvalidArgument(response, data)
 
-                if response.status == 403:
-                    if data.get("reason") in ["accessDenied.invalidIp"]:
-                        if not api_request:
-                            await self.reset_key(key)
-                            LOG.info("Reset Clash of Clans key")
-                            return await self.request(route, **kwargs)
+                        if response.status == 403:
+                            if data.get("reason") in ["accessDenied.invalidIp"]:
+                                if not api_request:
+                                    await self.reset_key(key)
+                                    LOG.info("Reset Clash of Clans key")
+                                    return await self.request(route, **kwargs)
 
-                    raise Forbidden(response, data)
+                            raise Forbidden(response, data)
 
-                if response.status == 404:
-                    raise NotFound(response, data)
-                if response.status == 429:
-                    LOG.error(
-                        "We have been rate-limited by the API. "
-                        "Reconsider the number of requests you are allowing per second."
-                    )
-                    raise HTTPException(response, data)
+                        if response.status == 404:
+                            raise NotFound(response, data)
+                        if response.status == 429:
+                            LOG.error(
+                                "We have been rate-limited by the API. "
+                                "Reconsider the number of requests you are allowing per second."
+                            )
+                            raise HTTPException(response, data)
 
-                if response.status == 503:
-                    if isinstance(data, str):
-                        # weird case where a 503 will be raised, but html returned.
-                        text = re.compile(r"<[^>]+>").sub(data, "")
-                        raise Maintenance(response, text)
+                        if response.status == 503:
+                            if isinstance(data, str):
+                                # weird case where a 503 will be raised, but html returned.
+                                text = re.compile(r"<[^>]+>").sub(data, "")
+                                raise Maintenance(response, text)
 
-                    raise Maintenance(response, data)
-                if response.status in [502, 504]:  # bad gateway, gateway timeout
-                    # gateway errors return html
+                            raise Maintenance(response, data)
+
+                        if response.status in (500, 502, 504):
+                            # gateway error, retry again
+                            await asyncio.sleep(tries * 2 + 1)
+                            continue
+
+            except asyncio.TimeoutError:
+                # api timed out, retry again
+                if tries > 3:
+                    raise GatewayError("The API timed out waiting for the request.")
+
+                await asyncio.sleep(tries * 2 + 1)
+                continue
+            raise
+
+        else:
+            if response.status in (500, 502, 504):
+                if isinstance(data, str):
+                    # gateway errors return HTML
                     text = re.compile(r"<[^>]+>").sub(data, "")
                     raise GatewayError(response, text)
 
-                raise HTTPException(response, data)
+                raise GatewayError(response, data)
 
-    async def get_ip(self):
-        url = "https://api.ipify.org/"
-        async with self.__session.request("GET", url) as response:
-            LOG.debug("%s (%s) has returned %s", url, "GET", response.status)
-            ip_ = await response.text()
-            LOG.debug("%s has received %s", url, ip_)
-        return ip_
-
-    @staticmethod
-    def create_cookies(response_dict, session):
-        try:
-            return "session={};game-api-url={};game-api-token={}".format(
-                session, response_dict["swaggerUrl"], response_dict["temporaryAPIToken"]
-            )
-        except KeyError:
-            return None
-
-    async def reset_key(self, key):
-        ip_ = await self.get_ip()
-        # should probably put something else in here
-        # to distinguish each key like a date
-        key_name = self.key_names
-        # Also, probably fix this as well
-        key_description = "Created on {}".format(datetime.now().strftime("%c"))
-        whitelisted_ips = [ip_]
-
-        response_dict, session = await self.login_to_site(self.email, self.password)
-        cookies = self.create_cookies(response_dict, session)
-
-        if cookies is None:
-            return  # same issue as few lines down explains apparently.
-
-        existing_keys_dict = await self.find_site_keys(cookies)
-        existing_keys = existing_keys_dict and existing_keys_dict.get("keys")
-        if existing_keys is None:
-            # long standing bug where the dev site doesn't give a proper return dict when
-            # multiple concurrent logins are made. this is just a safety net, hopefully one of
-            # the requests will work.
-            return
-
-        key_id = [t["id"] for t in existing_keys if t["key"] == key]
-
-        try:
-            await self.delete_key(cookies, key_id)
-        except InvalidArgument:
-            return
-
-        new_key = await self.create_key(cookies, key_name, key_description, whitelisted_ips, [self.key_scopes])
-
-        # this is to prevent reusing an already used keys.
-        # All it does is move the current key to the front,
-        # by moving any already used ones to the end so
-        # we keep the original key order moving forward.
-        keys = self._keys
-        key_index = keys.index(key)
-        self._keys = keys[key_index:] + keys[:key_index]
-
-        # now we can set the new key which is the first
-        # one in self._keys, then start the cycle over.
-        self._keys[0] = new_key
-        self.keys = cycle(self._keys)
-        self.client.dispatch("key_reset", new_key)
+            raise HTTPException(response, data)
 
     # clans
 
@@ -477,6 +394,9 @@ class HTTPClient:
     def get_player(self, player_tag):
         return self.request(Route("GET", "/players/{}".format(player_tag), {}))
 
+    def verify_player_token(self, player_tag, token):
+        return self.request(Route("POST", "/players/{}/verifytoken".format(player_tag), {}), json={"token": token})
+
     # labels
 
     def get_clan_labels(self, **kwargs):
@@ -487,6 +407,125 @@ class HTTPClient:
 
     # key updating management
 
+    async def get_ip(self):
+        url = "https://api.ipify.org/"
+        async with self.__session.request("GET", url) as response:
+            LOG.debug("%s (%s) has returned %s", url, "GET", response.status)
+            ip_ = await response.text()
+            LOG.debug("%s has received %s", url, ip_)
+        return ip_
+
+    @staticmethod
+    def create_cookies(response_dict, session):
+        try:
+            return "session={};game-api-url={};game-api-token={}".format(
+                session, response_dict["swaggerUrl"], response_dict["temporaryAPIToken"]
+            )
+        except KeyError:
+            return None
+
+    async def get_keys(self):
+        self.client._ready.clear()
+
+        response_dict, session = await self.login_to_site(self.email, self.password)
+        cookies = self.create_cookies(response_dict, session)
+
+        headers = {
+            "cookies": cookies,
+            "content-type": "application/json",
+        }
+
+        ip = await self.get_ip()
+        current_keys = (await self.find_site_keys(headers))["keys"]
+
+        self._keys = [key["key"] for key in current_keys if key["name"] == self.key_names and ip in key["cidrRanges"]]
+
+        required_key_count = self.key_count
+        current_key_count = len(current_keys)
+
+        if required_key_count > len(self._keys):
+            for key in (k for k in current_keys if k["name"] == self.key_names and ip not in k["cidrRanges"]):
+                try:
+                    await self.delete_key(cookies, key["id"])
+                except (InvalidArgument, NotFound):
+                    continue
+                else:
+                    new = await self.create_key(cookies)
+                    self._keys.append(new)
+                    self.client.dispatch("on_key_reset", new)
+
+            make_keys = required_key_count - len(self._keys)
+            for _ in range(make_keys):
+                if current_key_count >= KEY_MAXIMUM:
+                    break
+
+                new = await self.create_key(cookies)
+                self._keys.append(new)
+                self.client.dispatch("on_key_reset", new)
+                current_key_count += 1
+
+            if current_key_count == KEY_MAXIMUM and len(self._keys) < required_key_count:
+                LOG.critical("%s keys were requested to be used, but a maximum of %s could be "
+                             "found/made on the developer site, as it has a maximum of 10 keys per account. "
+                             "Please delete some keys or lower your `key_count` level."
+                             "I will use %s keys for the life of this client.",
+                             required_key_count, current_key_count, current_key_count)
+
+        if len(self._keys) == 0:
+            await self.close()
+            raise RuntimeError(
+                "There are {} API keys already created and none match a key_name of '{}'."
+                "Please specify a key_name kwarg, or go to 'https://developer.clashofclans.com' to delete "
+                "unused keys.".format(current_key_count, self.key_names)
+            )
+
+        self.keys = cycle(self._keys)
+        self.client._ready.set()
+
+    async def reset_key(self, key):
+        response_dict, session = await self.login_to_site(self.email, self.password)
+        cookies = self.create_cookies(response_dict, session)
+
+        if cookies is None:
+            return  # same issue as few lines down explains apparently.
+
+        headers = {
+            "cookies": cookies,
+            "content-type": "application/json",
+        }
+
+        existing_keys_dict = await self.find_site_keys(headers)
+        try:
+            existing_keys = existing_keys_dict["keys"]
+        except (TypeError, KeyError):
+            # long standing bug where the dev site doesn't give a proper return dict when
+            # multiple concurrent logins are made. this is just a safety net, hopefully one of
+            # the requests will work.
+            return
+
+        key_id = [t["id"] for t in existing_keys if t["key"] == key]
+
+        try:
+            await self.delete_key(cookies, key_id)
+        except (InvalidArgument, NotFound):
+            return
+
+        new_key = await self.create_key(cookies)
+
+        # this is to prevent reusing an already used keys.
+        # All it does is move the current key to the front,
+        # by moving any already used ones to the end so
+        # we keep the original key order moving forward.
+        keys = self._keys
+        key_index = keys.index(key)
+        self._keys = keys[key_index:] + keys[:key_index]
+
+        # now we can set the new key which is the first
+        # one in self._keys, then start the cycle over.
+        self._keys[0] = new_key
+        self.keys = cycle(self._keys)
+        self.client.dispatch("key_reset", new_key)
+
     async def login_to_site(self, email, password):
         login_data = {"email": email, "password": password}
         headers = {"content-type": "application/json"}
@@ -504,26 +543,25 @@ class HTTPClient:
 
         return response_dict, session
 
-    async def find_site_keys(self, cookies):
-        headers = {"cookie": cookies, "content-type": "application/json"}
-        async with self.__session.post(
-            "https://developer.clashofclans.com/api/apikey/list", data=json.dumps({}), headers=headers,
-        ) as sess:
+    async def find_site_keys(self, headers):
+        url = "https://developer.clashofclans.com/api/apikey/list"
+        async with self.__session.post(url, json={}, headers=headers) as sess:
             existing_keys_dict = await sess.json()
-            LOG.debug(
-                "%s has received %s", "https://developer.clashofclans.com/api/apikey/list", existing_keys_dict,
-            )
+            LOG.debug("%s has received %s", url, existing_keys_dict)
 
         return existing_keys_dict
 
-    async def create_key(self, cookies, key_name, key_description, cidr_ranges, scopes):
-        headers = {"cookie": cookies, "content-type": "application/json"}
+    async def create_key(self, cookies):
+        headers = {
+            "cookie": cookies,
+            "content-type": "application/json"
+        }
 
         data = {
-            "name": key_name,
-            "description": key_description,
-            "cidrRanges": cidr_ranges,
-            "scopes": scopes,
+            "name": self.key_names,
+            "description": "Created on {}".format(datetime.now().strftime("%c")),
+            "cidrRanges": [await self.get_ip()],
+            "scopes": [self.key_scopes],
         }
 
         response = await self.request(
