@@ -35,9 +35,12 @@ from .enums import (
     UNRANKED_LEAGUE_DATA,
     ACHIEVEMENT_ORDER,
     SUPER_TROOP_ORDER,
+    HERO_PETS_ORDER,
 )
 from .abc import BasePlayer
 from .player_clan import PlayerClan
+from .utils import cached_property
+
 
 if typing.TYPE_CHECKING:
     # pylint: disable=cyclic-import
@@ -57,12 +60,16 @@ class ClanMember(BasePlayer):
         The member's experience level.
     league: :class:`League`
         The member's current league.
+    trophies: :class:`int`
+        The member's trophy count.
     versus_trophies: :class:`int`
         The member's versus trophy count.
     clan_rank: :class:`int`
         The member's rank in the clan.
     clan_previous_rank: :class:`int`
-        The member's rank in the clan at the end of the last season.
+        The member's rank before the last leaderboard change
+        (ie if Bob overtakes Jim in trophies, and they switch ranks on the leaderboard,
+        and you want to find out their previous rankings, this will help.).
     donations: :class:`int`
         The member's donation count for this season.
     received: :class:`int`
@@ -141,9 +148,11 @@ class RankedPlayer(ClanMember):
     versus_trophies: :class:`int`
         The player's versus trophy count. If retrieving info for regular leader-boards, this will be ``None``.
     rank: :class:`int`
-        The player's rank in the leader board.
+        The player's rank in the clan leaderboard.
     previous_rank: :class:`int`
-        The player's rank in the previous season's leaderboard.
+        The member's rank before the last clan leaderboard change
+        (ie if Bob overtakes Jim in trophies, and they switch ranks on the leaderboard,
+        and you want to find out their previous rankings, this will help.).
     """
 
     __slots__ = ("attack_wins", "defense_wins", "versus_trophies", "rank", "previous_rank")
@@ -224,6 +233,18 @@ class Player(ClanMember):
         "label_cls",
         "spell_cls",
         "troop_cls",
+
+        "_cs_labels",
+        "_cs_achievements",
+        "_cs_troops",
+        "_cs_home_troops",
+        "_cs_builder_troops",
+        "_cs_siege_machines",
+        "_cs_hero_pets",
+        "_cs_super_troops",
+        "_cs_heroes",
+        "_cs_spells",
+
     )
 
     def __init__(self, *, data, client, **_):
@@ -231,7 +252,6 @@ class Player(ClanMember):
 
         self._achievements = None  # type: typing.Optional[dict]
         self._heroes = None  # type: typing.Optional[dict]
-        self._labels = None  # type: typing.Optional[list]
         self._spells = None  # type: typing.Optional[dict]
         self._home_troops = None  # type: typing.Optional[dict]
         self._builder_troops = None  # type: typing.Optional[dict]
@@ -267,31 +287,25 @@ class Player(ClanMember):
 
         self.__iter_labels = (label_cls(data=ldata, client=self._client) for ldata in data_get("labels", []))
         self.__iter_achievements = (achievement_cls(data=adata) for adata in data_get("achievements", []))
-        self.__iter_troops = (
-            troop_cls(data=tdata) for tdata in data_get("troops", []) if tdata.get("name") not in SUPER_TROOP_ORDER
-        )
+        self.__iter_troops = (troop_cls(data=tdata) for tdata in data_get("troops", []))
         self.__iter_heroes = (hero_cls(data=hdata) for hdata in data_get("heroes", []))
         self.__iter_spells = (spell_cls(data=sdata) for sdata in data_get("spells", []))
 
-    @property
+    def _inject_clan_member(self, member):
+        if member:
+            self.clan_rank = getattr(member, "clan_rank", None)
+            self.clan_previous_rank = getattr(member, "clan_previous_rank", None)
+
+    @cached_property("_cs_labels")
     def labels(self) -> typing.List[Label]:
         """List[:class:`Label`]: A :class:`List` of :class:`Label` that the player has."""
-        labels = self._labels
-        if labels is not None:
-            return labels
+        return list(self.__iter_labels)
 
-        labels = self._labels = list(self.__iter_labels)
-        return labels
-
-    @property
+    @cached_property("_cs_achievements")
     def achievements(self) -> typing.List[Achievement]:
         """List[:class:`Achievement`]: A list of the player's achievements."""
         # at the time of writing, the API presents achievements in the order
         # added to the game which doesn't match in-game order.
-        dict_achievements = self._achievements
-        if dict_achievements is not None:
-            return list(dict_achievements.values())
-
         achievement_dict = {a.name: a for a in self.__iter_achievements}
         sorted_achievements = {}
         for name in ACHIEVEMENT_ORDER:
@@ -328,23 +342,19 @@ class Player(ClanMember):
         except KeyError:
             return default_value
 
-    @property
+    @cached_property("_cs_troops")
     def troops(self) -> typing.List[Troop]:
         """List[:class:`Troop`]: A :class:`List` of the player's :class:`Troop`.
 
         Troops are **not** ordered in this attribute. Use either :attr:`Player.home_troops`
         or :attr:`Player.builder_troops` if you want an ordered list.
         """
-        dict_troops = self._home_troops
-        if dict_troops is not None:
-            return list(dict_troops.values()) + list(self._builder_troops.values())
-
         troops = list(self.__iter_troops)
         self._home_troops = {t.name: t for t in troops if t.is_home_base}
         self._builder_troops = {t.name: t for t in troops if t.is_builder_base}
         return troops
 
-    @property
+    @cached_property("_cs_home_troops")
     def home_troops(self) -> typing.List[Troop]:
         """List[:class:`Troop`]: A :class:`List` of the player's home-base :class:`Troop`.
 
@@ -353,13 +363,11 @@ class Player(ClanMember):
         order = {k: v for v, k in enumerate(HOME_TROOP_ORDER)}
 
         if not self._home_troops:
-            troops = list(self.__iter_troops)
-            self._home_troops = {t.name: t for t in troops if t.is_home_base}
-            self._builder_troops = {t.name: t for t in troops if t.is_builder_base}
+            _ = self.troops
 
         return list(sorted(self._home_troops.values(), key=lambda t: order.get(t.name, 0)))
 
-    @property
+    @cached_property("_cs_builder_troops")
     def builder_troops(self) -> typing.List[Troop]:
         """List[:class:`Troop`]: A :class:`List` of the player's builder-base :class:`Troop`.
 
@@ -368,13 +376,11 @@ class Player(ClanMember):
         order = {k: v for v, k in enumerate(BUILDER_TROOPS_ORDER)}
 
         if not self._builder_troops:
-            troops = list(self.__iter_troops)
-            self._home_troops = {t.name: t for t in troops if t.is_home_base}
-            self._builder_troops = {t.name: t for t in troops if t.is_builder_base}
+            _ = self.troops
 
         return list(sorted(self._builder_troops.values(), key=lambda t: order.get(t.name, 0)))
 
-    @property
+    @cached_property("_cs_siege_machines")
     def siege_machines(self) -> typing.List[Troop]:
         """List[:class:`Troop`]: A :class:`List` of the player's siege-machine :class:`Troop`.
 
@@ -384,6 +390,26 @@ class Player(ClanMember):
         troops = (t for t in self.troops if t.name in SIEGE_MACHINE_ORDER)
         return list(sorted(troops, key=lambda t: order.get(t.name, 0)))
 
+    @cached_property("_cs_hero_pets")
+    def hero_pets(self) -> typing.List[Troop]:
+        """List[:class:`Troop`]: A :class:`List` of the player's hero pets.
+
+        This will return hero pets in the order found in the Pet House in-game.
+        """
+        order = {k: v for v, k in enumerate(HERO_PETS_ORDER)}
+        troops = (t for t in self.troops if t.name in HERO_PETS_ORDER)
+        return list(sorted(troops, key=lambda t: order.get(t.name, 0)))
+
+    @cached_property("_cs_super_troops")
+    def super_troops(self) -> typing.List[Troop]:
+        """List[:class:`Troop`]: A :class:`List` of the player's super troops.
+
+        This will return super troops in the order found in the super troop boosting building, in game.
+        """
+        order = {k: v for v, k in enumerate(SUPER_TROOP_ORDER)}
+        troops = (t for t in self.troops if t.name in SUPER_TROOP_ORDER)
+        return list(sorted(troops, key=lambda t: order.get(t.name, 0)))
+
     def get_troop(self, name: str, is_home_troop=None, default_value=None) -> typing.Optional[Troop]:
         """Returns a troop with the given name.
 
@@ -419,16 +445,12 @@ class Player(ClanMember):
         except KeyError:
             return default_value
 
-    @property
+    @cached_property("_cs_heroes")
     def heroes(self) -> typing.List[Hero]:
         """List[:class:`Hero`]: A :class:`List` of the player's :class:`Hero`.
 
         This will return heroes in the order found in the store and labatory in-game.
         """
-        dict_heroes = self._heroes
-        if dict_heroes is not None:
-            return list(dict_heroes.values())
-
         heroes_dict = {h.name: h for h in self.__iter_heroes}
         sorted_heroes = {}
         for name in HERO_ORDER:
@@ -465,16 +487,13 @@ class Player(ClanMember):
         except KeyError:
             return default_value
 
-    @property
+    @cached_property("_cs_spells")
     def spells(self) -> typing.List[Spell]:
         """List[:class:`Spell`]: A :class:`List` of the player's :class:`Spell` ordered as they appear in-game.
 
         This will return spells in the order found in both spell factory and labatory in-game.
         """
-        dict_spells = self._spells
-        if dict_spells is None:
-            dict_spells = self._spells = {s.name: s for s in self.__iter_spells}
-
+        dict_spells = self._spells = {s.name: s for s in self.__iter_spells}
         order = {k: v for v, k in enumerate(SPELL_ORDER)}
         return list(sorted(dict_spells.values(), key=lambda s: order.get(s.name)))
 
