@@ -41,6 +41,10 @@ class ClanWar:
 
     Attributes
     ----------
+    clan:
+        :class:`WarClan`: The home clan.
+    opponent:
+        :class:`WarClan`: The opposition clan.
     state:
         :class:`str`: The clan's current war state.
     preparation_start_time:
@@ -86,12 +90,13 @@ class ClanWar:
         data_get = data.get
 
         self.state = data_get("state")
-        self.team_size = data_get("teamSize", 0)
         self.preparation_start_time = try_enum(Timestamp, data=data_get("preparationStartTime"))
         self.start_time = try_enum(Timestamp, data=data_get("startTime"))
         self.end_time = try_enum(Timestamp, data=data_get("endTime"))
         self.war_tag = data_get("tag")
 
+        self.team_size = data_get("teamSize") or len(data_get("clan", {}).get("members", []))
+
         clan_data = data_get("clan")
         # annoying bug where if you request a war with a clan tag that clan could be the opponent or clan,
         # depending on the way the game stores it internally. This isn't very helpful as we always want it
@@ -377,7 +382,7 @@ class ClanWarLeagueGroup:
 
     @property
     def clans(self) -> typing.List[ClanWarLeagueClan]:
-        """List[class:`LeagueClan`]: Returns all participating clans."""
+        """List[:class:`LeagueClan`]: Returns all participating clans."""
         clans = self._clans
         if clans:
             return clans
@@ -409,7 +414,7 @@ class ClanWarLeagueGroup:
 
         Returns
         ---------
-        AsyncIterator of :class:`ClanWar`: All wars in the given round.
+        All wars in the given round. : AsyncIterator of :class:`ClanWar`
         """
         return LeagueWarIterator(client=self._client, tags=itertools.chain(*self.rounds), clan_tag=clan_tag, cls=cls)
 
@@ -442,8 +447,9 @@ class ClanWarLeagueGroup:
 
 
         Returns
-        ---------
-        AsyncIterator of :class:`ClanWar`: All wars in the given round.
+        ----------
+
+        All wars in the given round. : AsyncIterator of :class:`ClanWar`
         """
         is_prep = self.state == "preparation"
         num_rounds = len(self.rounds)
